{*****************************************************}
{* 1-bit-full-adder unit for LogicGates3 application *}
{*****************************************************}

unit adder1f;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls, Grids, common;

type
  {***********}
  { TfAdder1f }
  {***********}
  TfAdder1f = class(TForm)
    Shape1: TShape;
    StaticText1: TStaticText;
    Label1, Label2, Label3, Label4, Label5, Label6, Label7: TLabel;
    Shape2, Shape3, Shape5, Shape6, Shape7, Shape8, Shape9, Shape10: TShape;
    Shape11, Shape12, Shape13, Shape14, Shape15, Shape16, Shape17: TShape;
    Shape19, Shape20, Shape22, Shape23, Shape24, Shape25, Shape26: TShape;
    Shape27, Shape28, Shape29, Shape30, Shape31, Shape33, Shape34, Shape39: TShape;
    Shape41, Shape42, Shape43, Shape44, Shape45, Shape46, Shape47: TShape;
    Shape48, Shape50, Shape51, Shape52, Shape53, Shape54: TShape;
    Shape55, Shape56, Shape57, Shape58, Shape59, Shape62, Shape63: TShape;
    Shape64, Shape66, Shape68, Shape69, Shape70, Shape72: TShape;
    Image1, Image2, Image3, Image4, Image5: TImage;
    imSwitchA, imSwitchB, imSwitchC: TImage;
    shSwitchA1, shSwitchA2, shSwitchB1, shSwitchB2, shSwitchC1, shSwitchC2: TShape;
    laS1, laC1, laS2, laC2: TLabel;
    shLedA, shLedB, shLedC, shLedS1, shLedC1: TShape;
    shLedS2, shLedC2, shLedSout, shLedCout: TShape;
    Memo1: TMemo;
    StringGrid1: TStringGrid;
    btClose: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure imSwitchAClick(Sender: TObject);
    procedure imSwitchBClick(Sender: TObject);
    procedure imSwitchCClick(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
  private
    bA, bB, bC: Boolean;
    aLeds: TLeds;
    bLeds: TStatus;
  end;

const
  // Switches description: Data = objects positions (transparent image, long bar shape, short bar shape), horiz. closing move, vert. closing move
  DSwitches: array[0..2] of TSwitchData = (
    (Name: 'SwitchA';
     Data: (104, 108, 111, -1, 0)),
    (Name: 'SwitchB';
     Data: (173, 178, 179, -1, 0)),
    (Name: 'SwitchC';
     Data: (13, 33, 16, 1, 0))
  );

var
  fAdder1f: TfAdder1f;

implementation

{$R *.lfm}

{***********}
{ TfAdder1f }
{***********}

{ Application start: Initialisation }

procedure TfAdder1f.FormCreate(Sender: TObject);

begin
  SetLength(aLeds, 9); SetLength(bLeds, 9);
  aLeds[0] := shLedA; aLeds[1] := shLedB; aLeds[2] := shLedC;
  aLeds[3] := shLedS1; aLeds[4] := shLedC1;
  aLeds[5] := shLedS2; aLeds[6] := shLedC2;
  aLeds[7] := shLedSout; aLeds[8] := shLedCout;
  laS1.Caption := 'S' + SUB_Digits[1]; laC1.Caption := 'C' + SUB_Digits[1];
  laS2.Caption := 'S' + SUB_Digits[2]; laC2.Caption := 'C' + SUB_Digits[2];
end;

{ Form show-up: Initialize 1-bit-full-adder simulation}

procedure TfAdder1f.FormActivate(Sender: TObject);

var
  I: Integer;

begin
  // All inputs at 0
  bA := False; bB := False; bC := False;
  // Set switches to "off" position
  SwitchOnOff(DSwitches, imSwitchA, shSwitchA1, shSwitchA2, bA);
  SwitchOnOff(DSwitches, imSwitchB, shSwitchB1, shSwitchB2, bB);
  SwitchOnOff(DSwitches, imSwitchC, shSwitchC1, shSwitchC2, bC);
  // Turn LEDs "off"
  for I := 0 to Length(bLeds) - 1 do
    bLeds[I] := False;
  LedsOnOff(aLeds, bLeds);
end;

{ Switch "A" / "B" / "C" activated: Do 1-bit full-adder simulation for actual switch positions }

procedure TfAdder1f.imSwitchAClick(Sender: TObject);

begin
  // Change input at A from 0 to Vs (and vice-versa)
  bA := not bA;
  // Led "A" status
  bLeds[0] := bA;
  // Set switch A position
  SwitchOnOff(DSwitches, imSwitchA, shSwitchA1, shSwitchA2, bA);
  // Do 1-bit-full-adder calculation
  Adder(bA, bB, bC, bLeds[3], bLeds[4], bLeds[5], bLeds[6], bLeds[7], bLeds[8]);
  // Turn LEDs on or off
  LedsOnOff(aLeds, bLeds);
end;

procedure TfAdder1f.imSwitchBClick(Sender: TObject);

begin
  // Do the same as above for input B
  bB := not bB;
  bLeds[1] := bB;
  SwitchOnOff(DSwitches, imSwitchB, shSwitchB1, shSwitchB2, bB);
  Adder(bA, bB, bC, bLeds[3], bLeds[4], bLeds[5], bLeds[6], bLeds[7], bLeds[8]);
  LedsOnOff(aLeds, bLeds);
end;

procedure TfAdder1f.imSwitchCClick(Sender: TObject);

begin
  // Do the same as above for input C
  bC := not bC;
  bLeds[2] := bC;
  SwitchOnOff(DSwitches, imSwitchC, shSwitchC1, shSwitchC2, bC);
  Adder(bA, bB, bC, bLeds[3], bLeds[4], bLeds[5], bLeds[6], bLeds[7], bLeds[8]);
  LedsOnOff(aLeds, bLeds);
end;

{ Button "Close": Close the window }

procedure TfAdder1f.btCloseClick(Sender: TObject);

begin
  Close;
end;

end.

