{*****************************************************}
{* 1-bit-half-adder unit for LogicGates3 application *}
{*****************************************************}

unit adder1h;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls, Grids, Buttons, common;

type
  {***********}
  { TfAdder1h }
  {***********}
  TfAdder1h = class(TForm)
    Shape1: TShape;
    StaticText1: TStaticText;
    Label1, Label2, Label3, Label4, Label5, Label6: TLabel;
    Shape2, Shape3, Shape5, Shape6, Shape7, Shape8, Shape9, Shape10: TShape;
    Shape11, Shape12, Shape13, Shape15, Shape16, Shape17: TShape;
    Shape19, Shape20, Shape22, Shape23, Shape24, Shape25, Shape26: TShape;
    Shape27, Shape28, Shape29, Shape30, Shape31, Shape33, Shape34: TShape;
    Image1, Image2: TImage;
    imSwitchA, imSwitchB: TImage;
    shSwitchA1, shSwitchA2, shSwitchB1, shSwitchB2: TShape;
    shLedA, shLedB, shLedS, shLedC: TShape;
    StringGrid1: TStringGrid;
    btClose: TButton;
    procedure FormActivate(Sender: TObject);
    procedure imSwitchAClick(Sender: TObject);
    procedure imSwitchBClick(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
  private
    bA, bB: Boolean;
  end;

const
  // Switches description: Data = objects positions (transparent image, long bar shape, short bar shape), horiz. closing move, vert. closing move
  DSwitches: array[0..1] of TSwitchData = (
    (Name: 'SwitchA';
     Data: (13, 33, 16, 1, 0)),
    (Name: 'SwitchB';
     Data: (114, 119, 120, -1, 0))
  );

var
  fAdder1h: TfAdder1h;

implementation

{$R *.lfm}

{***********}
{ TfAdder1h }
{***********}

{ Form show-up: Initialize 1-bit half-adder simulation }

procedure TfAdder1h.FormActivate(Sender: TObject);

begin
  // All inputs at 0V
  bA := False; bB := False;
  // Set switches to "off" position
  SwitchOnOff(DSwitches, imSwitchA, shSwitchA1, shSwitchA2, False); SwitchOnOff(DSwitches, imSwitchB, shSwitchB1, shSwitchB2, False);
  // Turn LEDs "off"
  LedOnOff(shLedA, False); LedOnOff(shLedB, False); LedOnOff(shLedS, False); LedOnOff(shLedC, False);
end;

{ Switch "A" / "B" activated: Do 1-bit half-adder simulation for actual switch positions }

procedure TfAdder1h.imSwitchAClick(Sender: TObject);

var
  S, C: Boolean;

begin
  // Change input at A from 0 to Vs (and vice-versa)
  bA := not bA;
  // Set switch A position
  SwitchOnOff(DSwitches, imSwitchA, shSwitchA1, shSwitchA2, bA);
  // Turn input LED A on or off
  LedOnOff(shLedA, bA); LedOnOff(shLedB, bB);
  // Do 1-bit-half-adder calculation
  HalfAdder(bA, bB, S, C);
  // Turn output LEDs on or off
  LedOnOff(shLedS, S); LedOnOff(shLedC, C);
end;

procedure TfAdder1h.imSwitchBClick(Sender: TObject);

var
  S, C: Boolean;

begin
  // Do the same as above for input B
  bB := not bB;
  SwitchOnOff(DSwitches, imSwitchB, shSwitchB1, shSwitchB2, bB);
  LedOnOff(shLedA, bA); LedOnOff(shLedB, bB);
  HalfAdder(bA, bB, S, C);
  LedOnOff(shLedS, S); LedOnOff(shLedC, C);
end;

{ Button "Close": Close the window }

procedure TfAdder1h.btCloseClick(Sender: TObject);

begin
  Close;
end;

end.

